package waste

import (
	"context"
	"fmt"
	"strconv"
	"strings"

	"git.3crabs.ru/save_my_money/smm_core/internal/services/category"
	"github.com/jackc/pgx/v5"
	"github.com/jackc/pgx/v5/pgxpool"
	"golang.org/x/text/cases"
	"golang.org/x/text/language"
)

type WasteEntity struct {
	Id           int
	Name         string
	Price        int
	Amount       float32
	CategoryId   int
	CategoryName string
	BudgetId     int
}

type WasteTextEntity struct {
	Text       string
	CategoryId int
	BudgetId   int
}

type WasteService struct {
	db              *pgxpool.Pool
	categoryService *category.CategoryService
}

func NewWasteService(
	db *pgxpool.Pool,
	categoryService *category.CategoryService,
) *WasteService {
	return &WasteService{
		db:              db,
		categoryService: categoryService,
	}
}

func (s *WasteService) AddWaste(ctx context.Context, waste *WasteEntity) (*WasteEntity, error) {
	query := `INSERT INTO wastes (name, price, amount, budget_id, category_id) VALUES (@name, @price, @amount, @budget_id, @category_id) RETURNING id`
	args := pgx.NamedArgs{
		"name":        waste.Name,
		"price":       waste.Price,
		"amount":      waste.Amount,
		"budget_id":   waste.BudgetId,
		"category_id": waste.CategoryId,
	}
	if err := s.db.QueryRow(ctx, query, args).Scan(&waste.Id); err != nil {
		return nil, fmt.Errorf("unable to insert row: %w", err)
	}
	return waste, nil
}

func (s *WasteService) GetWaste(ctx context.Context, budgetId int, name string) (*WasteEntity, error) {
	query := `SELECT id, name, price, amount, budget_id, category_id FROM wastes WHERE budget_id = @budget_id AND LOWER(name) = LOWER(@name) LIMIT 1`
	args := pgx.NamedArgs{
		"name":      name,
		"budget_id": budgetId,
	}
	rows, err := s.db.Query(ctx, query, args)
	if err != nil {
		return nil, err
	}
	wastes := []*WasteEntity{}
	defer rows.Close()
	for rows.Next() {
		waste := &WasteEntity{}
		err = rows.Scan(&waste.Id, &waste.Name, &waste.Price, &waste.Amount, &waste.BudgetId, &waste.CategoryId)
		if err != nil {
			return nil, err
		}
		wastes = append(wastes, waste)
	}
	if len(wastes) > 0 {
		return wastes[0], nil
	}
	return nil, nil
}

func (s *WasteService) AddWasteByText(ctx context.Context, wasteText *WasteTextEntity) (*WasteEntity, error) {
	waste := s.parseWaste(wasteText.Text)

	var categoryId *int
	if waste.CategoryName != "" {
		c, err := s.categoryService.GetCategory(ctx, wasteText.BudgetId, waste.CategoryName)
		if err != nil {
			return nil, err
		}
		if c == nil {
			c, err = s.categoryService.AddCategory(ctx, &category.CategoryEntity{
				Name:     waste.CategoryName,
				BudgetId: int(wasteText.BudgetId),
			})
			if err != nil {
				return nil, err
			}
		}
		categoryId = &c.Id
	}
	if categoryId == nil {
		w, err := s.GetWaste(ctx, wasteText.BudgetId, waste.Name)
		if err != nil {
			return nil, err
		}
		if w != nil {
			categoryId = &w.CategoryId
		}
	}

	if categoryId == nil {
		return nil, fmt.Errorf("bad request")
	}

	query := `INSERT INTO wastes (name, price, amount, budget_id, category_id) VALUES (@name, @price, @amount, @budget_id, @category_id) RETURNING id`
	args := pgx.NamedArgs{
		"name":        waste.Name,
		"price":       waste.Price,
		"amount":      waste.Amount,
		"budget_id":   wasteText.BudgetId,
		"category_id": *categoryId,
	}
	if err := s.db.QueryRow(ctx, query, args).Scan(&waste.Id); err != nil {
		return nil, fmt.Errorf("unable to insert row: %w", err)
	}
	return waste, nil
}

func (s *WasteService) parseWaste(text string) *WasteEntity {
	text = strings.TrimSpace(text)
	arr := strings.Split(text, ":")

	waste := &WasteEntity{}
	switch len(arr) {
	case 1:
		arr = strings.Split(strings.TrimSpace(arr[0]), " ")
	case 2:
		waste.CategoryName = strToName(arr[0])
		arr = strings.Split(strings.TrimSpace(arr[1]), " ")
	}

	switch len(arr) {
	case 2:
		waste.Name = strToName(arr[0])
		waste.Amount = 1
		waste.Price = strToPrice(arr[1])
	case 3:
		waste.Name = strToName(arr[0])
		waste.Amount = strToAmount(arr[1])
		waste.Price = strToPrice(arr[2])
	}
	return waste
}

func strToFloat(s string) float32 {
	a, err := strconv.ParseFloat(s, 32)
	if err != nil {
		panic(err)
	}
	return float32(a)
}

func strToPrice(s string) int {
	s = strings.TrimSpace(s)
	s = strings.TrimSuffix(s, "/кг")
	s = strings.TrimSuffix(s, "/шт")
	s = strings.TrimSuffix(s, "р")
	s = strings.TrimSuffix(s, "руб")
	s = strings.TrimSuffix(s, "рублей")
	return int(strToFloat(s) * 100)
}

func strToAmount(s string) float32 {
	s = strings.TrimSpace(s)
	s = strings.TrimSuffix(s, "шт")
	s = strings.TrimSuffix(s, "кг")
	return strToFloat(s)
}

func strToName(s string) string {
	caser := cases.Title(language.Russian)
	return caser.String(strings.TrimSpace(strings.ToLower(s)))
}
